/**************************************************************************
 *									  *
 *    File "com_slave.c"						  *
 *									  *
 *    Simple Matrix Multiplication for P4				  *
 *									  *
 *    Master program in  "com_slave.c"					  *
 *    Process Group File "com.pg" 					  *
 *									  *
 *    Note that this program must be linked with "p4_cmain.o" in lib dir. *
 **************************************************************************/

#include "p4.h"
   
#define DATA  1
#define MAX   50
 
/*-------------------------------------------------------------------*/

slave()
{
  int i, j, type, to, from, size, my_id, matrix_size, load;
  int *incoming, A[MAX][MAX], B[MAX], result[MAX];

  my_id = p4_get_my_id();

  /* receive the data from master */
  type = DATA;
  size = sizeof(int);
  from = 0;  /* master process */

  incoming = NULL;
  p4_recv(&type, &from, &incoming, &size);
  matrix_size = *incoming;
  p4_msg_free(incoming);

  incoming = NULL;
  p4_recv(&type, &from, &incoming, &size);
  load = *incoming;
  p4_msg_free(incoming);

  printf("\n slave %d: ",my_id);
  printf("matrix_size = %d     load = %d\n", matrix_size, load);

  size = matrix_size * sizeof(int);

  /* receive part of matrix A */
  for ( i = 0 ; i < load ; i++ ) {
    incoming = NULL;
    p4_recv(&type, &from, &incoming, &size);
    for ( j = 0 ; j < matrix_size ; j++ ) {
      A[i][j] = incoming[j];
    }
    p4_msg_free(incoming);
  }

  incoming = NULL;
  p4_recv(&type, &from, &incoming, &size);
  for ( j = 0 ; j < matrix_size ; j++ ) {
    B[j] = incoming[j];
  }
  p4_msg_free(incoming);

  printf("\n slave %d: Received A= \n",my_id);
  for ( i = 0 ; i < load ; i++) {
    for ( j = 0 ; j < matrix_size ; j++ ) {
      printf("  %d  ", A[i][j]);
    }
    printf("\n");
  }

  printf("\n slave %d: Received B= \n",my_id);
  for ( i = 0 ; i < matrix_size ; i++ )
    printf("  %d  ", B[i]);
  printf("\n");

  /* compute the allocated data */
  for ( i = 0 ; i < load ; i++ ) {
    result[i] = 0; 
    for ( j = 0 ; j < matrix_size ; j++ ) {
      result[i] += A[i][j] * B[j];
    }
  }

  to = 0;   /* master */
  size = load * sizeof(int);
  p4_sendr(DATA, to, result, size);
  printf("\n slave %d: is sending the result\n");
  for ( i = 0 ; i < load ; i++ )
    printf("  %d  ",result[i]);
  printf("\n");

} /* end of slave */





